// -----------------------------------------------------------------------------
// spell_aoe_instant
// -----------------------------------------------------------------------------
/*
    Script for Area of Effect spells that have an instant effect.
*/
// -----------------------------------------------------------------------------
// petert
// -----------------------------------------------------------------------------

#include "log_h"
#include "abi_templates"
#include "combat_h"
#include "monster_constants_h"

void _ApplySpellEffects(struct EventSpellScriptImpactStruct stEvent, object oTarget)
{
    float fScaledValue;
    effect eEffect;

    switch (stEvent.nAbility)
    {
        case ABILITY_SPELL_MONSTER_OGRE_HURL:
        {
            if (stEvent.oCaster != oTarget)
            {
                float fDamage = (50.0f + GetAttributeModifier(stEvent.oCaster, PROPERTY_ATTRIBUTE_STRENGTH)) * OGRE_HURL_DAMAGE_FACTOR;
                eEffect = EffectDamage(fDamage);
                ApplyEffectOnObject(EFFECT_DURATION_TYPE_INSTANT, eEffect, oTarget, 0.0f, stEvent.oCaster, stEvent.nAbility);

                // physical resistance
                if (ResistanceCheck(stEvent.oCaster, oTarget, PROPERTY_ATTRIBUTE_STRENGTH, RESISTANCE_PHYSICAL) == FALSE)
                {
                    // knockdown
                    eEffect = EffectKnockdown(oTarget, 0, stEvent.nAbility);
                    eEffect = SetEffectEngineInteger(eEffect, EFFECT_INTEGER_USE_INTERPOLATION_ANGLE, 2);
                    eEffect = SetEffectEngineVector(eEffect, EFFECT_VECTOR_ORIGIN, GetPositionFromLocation(stEvent.lTarget));
                    ApplyEffectOnObject(EFFECT_DURATION_TYPE_TEMPORARY, eEffect, oTarget, RandomFloat(), stEvent.oCaster, stEvent.nAbility);
                }
            }

            break;
        }
    }
}

// Spellscript Impact Damage and Effects
void _ApplyImpactDamageAndEffects(struct EventSpellScriptImpactStruct stEvent)
{
    // only work for sphere spells
    int nAoEType = GetM2DAInt(TABLE_ABILITIES_SPELLS, "aoe_type", stEvent.nAbility);
    if (nAoEType == 1)
    {
        // location impact vfx
        if (stEvent.oTarget != OBJECT_INVALID)
        {
            stEvent.lTarget = GetLocation(stEvent.oTarget);
        }
        Ability_ApplyLocationImpactVFX(stEvent.nAbility, stEvent.lTarget);

        float fRadius = GetM2DAFloat(TABLE_ABILITIES_SPELLS, "aoe_param1", stEvent.nAbility);

        // get objects in area of effect
        object[] oTargets = GetObjectsInShape(OBJECT_TYPE_CREATURE, SHAPE_SPHERE, stEvent.lTarget, fRadius);

        // spell-specific special events
        switch (stEvent.nAbility)
        {
            case ABILITY_SPELL_GOLEM_HURL:
            {
                string sTag = GetTag(GetArea(stEvent.oTarget));
                if ((sTag == "cir310ar_fade_burning_tower") || (sTag == "cir320ar_fade_darkspawn_inv") || (sTag == "cir330ar_fade_mage_asunder") || (sTag == "cir340ar_fade_templar_night"))
                {
                    event ev = Event(EVENT_TYPE_CUSTOM_EVENT_01);
                    SignalEvent(stEvent.oTarget, ev);
                }

                break;
            }
        }

        // cycle through objects
        int nCount = 0;
        int nMax = GetArraySize(oTargets);
        for (nCount = 0; nCount < nMax; nCount++)
        {
            // per-spell effects
            _ApplySpellEffects(stEvent, oTargets[nCount]);
        }
    }
}

void main()
{
    event ev = GetCurrentEvent();
    int nEventType = GetEventType(ev); // CAST (directly from engine) or COMMAND_PENDING (re-directed by rules_core)

    switch(nEventType)
    {
        case EVENT_TYPE_SPELLSCRIPT_PENDING:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptCastStruct stEvent = Events_GetEventSpellScriptCastParameters(ev);

            Log_Trace(LOG_CHANNEL_COMBAT_ABILITY, GetCurrentScriptName() + ".EVENT_TYPE_SPELLSCRIPT_PENDING", Log_GetAbilityNameById(stEvent.nAbility));

            // Setting Return Value
            Ability_SetSpellscriptPendingEventResult(COMMAND_RESULT_SUCCESS);

            if(stEvent.nAbility == ABILITY_SPELL_MONSTER_OGRE_HURL)
                ApplyEffectVisualEffect(OBJECT_SELF, OBJECT_SELF, 3000, EFFECT_DURATION_TYPE_TEMPORARY, 2.5, stEvent.nAbility);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_CAST:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptCastStruct stEvent = Events_GetEventSpellScriptCastParameters(ev);

            Log_Trace(LOG_CHANNEL_COMBAT_ABILITY, GetCurrentScriptName() + ".EVENT_TYPE_SPELLSCRIPT_CAST",Log_GetAbilityNameById(stEvent.nAbility));

            // hand this through to cast_impact
            SetAbilityResult(stEvent.oCaster, stEvent.nResistanceCheckResult);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_IMPACT:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptImpactStruct stEvent = Events_GetEventSpellScriptImpactParameters(ev);

            Log_Trace(LOG_CHANNEL_COMBAT_ABILITY, GetCurrentScriptName() + ".EVENT_TYPE_SPELLSCRIPT_IMPACT",Log_GetAbilityNameById(stEvent.nAbility));

            // AbiAoEImpact(stEvent.lTarget,  stEvent.oCaster, stEvent.nAbility);
            _ApplyImpactDamageAndEffects(stEvent);

            break;
        }
    }
}